; ==================================================================================================
; Title:   Splash.inc
; Author:  G. Friedrich
; Version: 1.0.0
; Purpose: Support of Splash windows for ObjAsm32.
; Notes:   Version 1.0.0, March 2006
;            - First release.
; ==================================================================================================


Object Splash, SplashID, Window
  RedefineMethod    Done
  RedefineMethod    Init,             Pointer, Pointer, dword
  StaticMethod      OnPaint,          dword, dword
  BoundMethod       Startup
  BoundMethod       Worker

  Event   OnPaint,    WM_PAINT

  DefineVariable    BmpData,          BITMAP,   {}
  DefineVariable    hPrvBmp,          Handle,   0
  DefineVariable    hMemDC,           Handle,   0
  DefineVariable    hSkinRegion,      Handle,   0
  DefineVariable    hThread,          Handle,   0
ObjectEnd


; ==================================================================================================

if IMPLEMENT

CStr szSplash, "Splash"

.code

; 
; Method:    Splash.Done
; Purpose:   Finalizes the Splash object, freeing allocated resources.
; Arguments: None.
; Return:    Nothing.

Method Splash.Done, uses esi
    SetObject esi

    ;Start fade out of the splash window
    invoke ResumeThread, [esi].hThread
    invoke WaitForSingleObject, [esi].hThread, 15000
    
    ;Housekeeping
    invoke CloseHandle, [esi].hThread
    invoke DeleteObject, $invoke(SelectObject, [esi].hMemDC, [esi].hPrvBmp)
    invoke DeleteDC, [esi].hMemDC
    ACall esi.Done
MethodEnd

; 
; Method:    Splash.Init
; Purpose:   Initalizes the Splash object.
; Arguments: Arg1: -> Owner object.
;            Arg2: -> BMP resource name.
;            Arg3: RGB transparent BMP color.
; Return:    Nothing.

Method Splash.Init, uses esi, pOwner:Pointer, pBmpResName:Pointer, dTranparentColor:dword
    local hBmp:Handle, hScreenDC:Handle

    SetObject esi
    ACall esi.Init, pOwner, NULL

    ;Load resource
    mov hBmp, $invoke(LoadBitmap, hInstance, pBmpResName)
    invoke GetObject, hBmp, sizeof BITMAP, addr [esi].BmpData

    ;Create a region of the bitmap to skin the window
    mov hScreenDC, $invoke(GetDC, 0)
    mov [esi].hMemDC, $invoke(CreateCompatibleDC, hScreenDC)
    invoke ReleaseDC, 0, hScreenDC
    mov [esi].hPrvBmp, $invoke(SelectObject, [esi].hMemDC, hBmp)
    invoke CreateRectRgn, 0, 0, 0, 0
    mov [esi].hSkinRegion, $invoke(Bmp2Rgn, [esi].hMemDC, eax, RGN_OR, dTranparentColor)

    ;Create worker thread to fade in the splash window
    invoke CreateThread, NULL, 0, $MethodAddr(Splash.Worker), esi, CREATE_SUSPENDED, NULL
    mov [esi].hThread, eax
    invoke SetThreadPriority, [esi].hThread, THREAD_PRIORITY_TIME_CRITICAL
    invoke ResumeThread, [esi].hThread
    
    ;Now resume the main initialization while the splash window is shown
MethodEnd

; 
; Method:    Splash.OnPaint
; Purpose:   Event procedure for WM_PAINT message.
; Arguments: Arg1: First message parameter.
;            Arg2: Second message parameter.
; Return:    Zero if handled.

Method Splash.OnPaint, uses esi, wParam:dword, lParam:dword
    local PS:PAINTSTRUCT

    SetObject esi
    invoke BeginPaint, [esi].hWnd, addr PS
    invoke BitBlt, eax, 0, 0, [esi].BmpData.bmWidth, [esi].BmpData.bmHeight, \
                   [esi].hMemDC, 0, 0, SRCCOPY
    invoke EndPaint, [esi].hWnd, addr PS
    xor eax, eax
MethodEnd

; 
; Method:    Splash.Startup
; Purpose:   Registers the object with the OS.
; Arguments: None.
; Return:    Nothing.

Method Splash.Startup
    local WC:WNDCLASSEX

    mov WC.cbSize, sizeof WNDCLASSEX
    mov WC.style, CS_SAVEBITS or CS_BYTEALIGNCLIENT
    m2m WC.lpfnWndProc, $MethodAddr(Splash.WndProc)
    mov WC.cbClsExtra, 0
    mov WC.cbWndExtra, 0
    m2m WC.hInstance, hInstance
    mov WC.hbrBackground, 0
    mov WC.lpszMenuName, NULL
    mov WC.lpszClassName, offset szSplash
    mov WC.hIcon, 0
    mov WC.hIconSm, 0
    mov WC.hCursor, $invoke(LoadCursor, NULL, IDC_WAIT)

    invoke RegisterClassEx, addr WC
MethodEnd

; 
; Method:    Splash.Worker
; Purpose:   Worker thread called by Init method.
; Arguments: None.
; Return:    Nothing.

Method Splash.Worker, uses esi
    local dPosX:dword, dPosY:dword

    SetObject esi, Splash

    ;Window creation
    mov dPosX, $invoke(CenterForm, [esi].BmpData.bmWidth, $invoke(GetSystemMetrics, SM_CXSCREEN))
    mov dPosY, $invoke(CenterForm, [esi].BmpData.bmHeight, $invoke(GetSystemMetrics, SM_CYSCREEN))
    invoke CreateWindowEx, WS_EX_TOOLWINDOW or WS_EX_TOPMOST or WS_EX_LAYERED, \
               offset szSplash, offset szSplash, WS_POPUP, \
               dPosX, dPosY, [esi].BmpData.bmWidth, [esi].BmpData.bmHeight, 0, 0, hInstance, esi
    invoke SetWindowRgn, [esi].hWnd, [esi].hSkinRegion, FALSE ;The region is now owned by the window!

    ;Fade in/out
    invoke WndFadeIn, [esi].hWnd, 0, 255, 15, 20
    invoke SuspendThread, [esi].hThread                   ;Wait until Done is called
    invoke WndFadeOut, [esi].hWnd, 255, 0, 15, 20

    ;Window destruction
    invoke DestroyWindow, [esi].hWnd
MethodEnd

endif